<?php
/**
 * Copyright ETS Software Technology Co., Ltd
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 * You are not allowed to redistribute, resell, lease, license, sub-license or offer our resources to any third party.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future.
 *
 * @author ETS Software Technology Co., Ltd
 * @copyright  ETS Software Technology Co., Ltd
 * @license    Valid for 1 website (or project) for each purchase of license
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class EtsRVProductCommentOrderLicense extends ObjectModel
{
    public $id_ets_rv_product_comment;
    public $id_license;
    public $id_product;

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'ets_rv_product_comment_order_license',
        'primary' => '',
        'multilang' => false,
        'fields' => array(
            'id_ets_rv_product_comment' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'required' => true),
            'id_license' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'id_product' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'size' => 500),
        ),
    );

    public static function getOldestOrderLicense($id_product)
    {
        if (!$id_product || !Validate::isUnsignedInt($id_product)) {
            return false;
        }
        $dq = new DbQuery();
        $dq
            ->select('op.id_license')
            ->from('ets_sl_order_product', 'op')
            ->leftJoin('ets_rv_product_comment_order_license', 'ol', 'ol.id_license=op.id_license AND op.id_product = ol.id_product')
            ->where('op.id_product=' . (int)$id_product)
            ->where('ol.id_product is NULL OR ol.id_product <= 0');

        return (int)Db::getInstance()->getValue($dq);
    }

    public static function getReviewed($id_license, $id_product)
    {
        if (!$id_product ||
            !Validate::isUnsignedInt($id_product) ||
            !$id_license ||
            !Validate::isUnsignedInt($id_license)
        ) {
            return false;
        }
        $dq = new DbQuery();
        $dq
            ->select('id_ets_rv_product_comment')
            ->from('ets_rv_product_comment_order_license', 'ol')
            ->where('ol.id_product = ' . (int)$id_product)
            ->where('ol.id_license = ' . (int)$id_license);

        return (int)Db::getInstance()->getValue($dq);
    }

    public static function saveData($multi_data = array())
    {
        if (!is_array($multi_data) || !count($multi_data)) {
            return false;
        }
        return Db::getInstance()->insert(self::$definition['table'], $multi_data, true, true, Db::INSERT_IGNORE);
    }

    public static function deleteReviewed($product_comment_id)
    {
        if (!$product_comment_id || !Validate::isUnsignedInt($product_comment_id)) {
            return false;
        }

        return Db::getInstance()->delete('ets_rv_product_comment_order_license', 'id_ets_rv_product_comment=' . (int)$product_comment_id);
    }


    public static function isOrderLicense()
    {
        return Db::getInstance()->executeS('SHOW TABLES LIKE \'' . _DB_PREFIX_ . 'ets_sl_order_product\'');
    }

    public static function getOrderLicense($id_license)
    {
        if (!$id_license || !Validate::isUnsignedInt($id_license)) {
            return false;
        }
        return Db::getInstance()->getRow('SELECT * FROM `' . _DB_PREFIX_ . 'ets_sl_order_product` WHERE `id_license`=' . (int)$id_license);
    }
}